/**
 * Projet Système de particules
 *
 * espace.c
 *
 * Implémentation du type espace
 *
 */
#include <stdlib.h>
#include <math.h>
#include "list.h"
#include "geometrie.h"
#include "particule.h"
#include "espace.h"

/** Fonction d'ajout d'un espace
 * Arguments:
 *	p1: Première particule
 *	p2: Seconde particule
 *	l: Longueur de l'espace
 * Retourne: L'espace qui vient d'être créé
 */
Espace* creer_espace(Particule* p1, Particule* p2, double l) {
	/* Allocation de la mémoire */
	Espace* espace = (Espace*)malloc(sizeof(Espace));

	/* Affectation des membres */
	espace -> e1 = p1;
	espace -> e2 = p2;

	espace -> longueur = l;

	return espace;
}

/** Procédure de résolution de contrainte de type "espace"
 * Argument: L'espace dont on veut résoudre la contrainte
 */
void resoudre_contrainte_espace(Espace* espace) {
	Point* pt1 = espace -> e1 -> position;
	Point* pt2 = espace -> e2 -> position;

	/* On récupère l'inverse des masses des particules */
	double invm1 = espace -> e1 -> masse;
	double invm2 = espace -> e2 -> masse; 
	
	/* Variables servant pour les calculs */
	double delta_x, delta_y;
	double delta;
	double diff;

	/* On ne modifie la distance que si les deux particules sont éloignées
	 * de moins de la longueur de l'espace.
	 */
	if(distance(pt1,pt2) < espace -> longueur) {
		

		/* On calcule la distance entre les deux points */
		delta_x = pt2 -> x - pt1 -> x;
		delta_y = pt2 -> y - pt1 -> y;

		/* delta est la norme du vecteur pt2 - pt1 */
		delta = sqrt(pow(delta_x,2)+pow(delta_y,2));

		/* La correction à appliquer aux points pour donner longueur fixée */
		diff = (delta - espace -> longueur)/(delta*(invm1 + invm2));

		/* On corrige le premier point */
		pt1 -> x += invm1*delta_x*diff;
		pt1 -> y += invm1*delta_y*diff;

		/* On corrige le second point */
		pt2 -> x -= invm2*delta_x*diff;
		pt2 -> y -= invm2*delta_y*diff;
	}
}

/** Procédure détruisant l'espace
 * Argument: espace: Espace à détruire
 */
void detruire_espace(Espace** espace) {
	free(*espace); (*espace) = NULL;
}

/** Procédure utilisée pour convertir l'appel à detruire_espace dans
 * l'itérateur
 * Argument: espace: L'espace à détruire
 */
void detruire_espace_it(Espace* espace) {
	detruire_espace(&espace);
}
