/*
 * Projet Système de particules
 *
 * interaction.c
 *
 * Fonctions de manipulation de l'IHM
 *
 */

#include <stdio.h>
#include <math.h>
#include "interaction.h"
#include "list.h"
#include "geometrie.h"
#include "particule.h"
#include "ancre.h"
#include "tige.h"
#include "systeme.h"
#include "parseur.h"

/** Si la macro CONTRAINTE est définie, alors on compile en mode "respect des
 * contraintes lors du déplacement. (voir plus bas, fonction
 * on_viewport_mouse_drag
 */
#define CONTRAINTE


/** Pointeur global vers le systeme déclaré dans le fichier
 * "visualiseur.c". On partage ainsi le même objet
*/
extern Systeme *systeme;

void on_viewport_mouse_press(double x, double y){
  /* Le pointeur sur l'ancre la plus proche */
  Ancre* ancre;

  /* Le pointeur sur la particule la plus proche */
  Particule* part;

  /* On crée des éléments temporaires servant à la recherche */
  Ancre* a_tmp = creer_ancre(NULL,x,y);
  Particule* p_tmp = creer_particule(x,y,0,-1);

  printf ("Bouton enfoncé: (%f, %f)\n", x, y);

  /** Recherche de l'ancre ou de la particule la plus proche de la souris **/

  /* ancre pointe sur l'ancre la plus proche (si elle existe) */
  ancre = member(a_tmp,systeme -> ancres, egalite_ancre_pos);

	
  /* On regarde si une ancre a été trouvée */
  if(ancre != NULL){
	  /* Si une ancre se trouve à proximité */
	  printf("Saisie de l'ancre associée à la particule n°%d\n",ancre -> particule -> id);
	  /* On l'ajoute comme ancre déplaçable */
	  systeme -> deplace = ancre;
	  /* On indique qu'il ne faudra pas la détruire */
	  systeme -> destructible = 0;
  } else {

	  /* part pointe sur la particule la plus proche (si elle existe)*/
	  part = member(p_tmp,systeme -> particules, egalite_particule_pos);
	  
	  /* On regarde si une particule a été trouvée */
	  if(part != NULL){
		  /* Si une particule se trouve à proximité */
		  printf("Saisie de la particule n°%d\n", part -> id);
		  /* Elle est ajoutée comme ancre dans le système */
		  systeme -> deplace = creer_ancre(part, x, y);
		  /* On indique qu'elle peut être détruite */
		  systeme -> destructible = 1;
	  }
  }

  /* Destruction des éléments temporaires */
  detruire_particule(&p_tmp);
  detruire_ancre(&a_tmp);


}

void on_viewport_mouse_release(){
  
  printf ("Bouton relaché\n");
  
  /* Il faut détruire l'ancre */
  if((systeme -> destructible) && (systeme -> deplace != NULL)){
  	detruire_ancre(&(systeme -> deplace));
  }

  /* On ne déplace plus l'ancre */
  systeme -> deplace = NULL;

}

/** Remarque :
 * 	Il y a deux "modes", choisies par une compilation conditionnelle
 * 	 -> Le premier (constante CONTRAINTE non-définie) n'impose aucun respect
 * des contraintes lors du déplacement. Le système est donc plus fluide.
 * Cependant, on subit un effet "elastique".
 * 	 -> Le second (constante CONTRAINTE définie) empêche l'utilisateur de
 * tirer une particule sans respecter les contraintes qui lui sont appliquées.
 * Cependant, la fluidité de la simulation est altérée.
 */
void on_viewport_mouse_drag(double orig_x, double orig_y,
                            double dest_x, double dest_y){
  #ifdef CONTRAINTE
  Ancre* tmp;
  #endif

  printf ("Cliqué glisser de (%f,%f) - (%f,%f)\n",
		    orig_x, orig_y, dest_x, dest_y);

  /* Si une particule a été trouvée */
  if(systeme -> deplace != NULL){
	
	#ifdef CONTRAINTE
	/** "Mode respect des contraintes pendant le déplacement" **/

	/* On MAJ la position de la particule directement */
	systeme -> deplace -> particule -> position -> x = dest_x;
	systeme -> deplace -> particule -> position -> y = dest_y;

	/* Si la particule cliquée était déjà une ancre,
	 * on déplace l'ancre aussi
	 */
	if(!(systeme -> destructible)) {
		systeme -> deplace -> position -> x = dest_x;
		systeme -> deplace -> position -> y = dest_y;
	}

	/* On garde un pointeur sur l'ancre pointée */
	tmp = systeme -> deplace;

	/* On "annule" temporairement la contrainte ancre */
	systeme -> deplace = NULL;

	/* Résolution des contraintes -> la particule pointée est donc ramenée à
	 * sa position correcte, dans le respect des contraintes */
	verlet(systeme);
	satisfaire_contraintes(systeme);

	/* On réinscrit l'ancre dans le système */
	systeme -> deplace = tmp;
	
	/* Puis mise à jour des coordonnées de l'ancre à l'aide des coordonées
	 * de la particule
	 * 	-> Pas très très beau
	 */
	systeme -> deplace -> position -> x = systeme -> deplace -> particule -> position -> x;
	systeme -> deplace -> position -> y = systeme -> deplace -> particule -> position -> y;
	
	#else
	/** Mode sans respect des contraintes pendant le déplacement */
	/* On ne déplace que l'ancre, la particule sera déplacée lors de la
	 * résolution des contraintes
	 */
		systeme -> deplace -> position -> x = dest_x;
		systeme -> deplace -> position -> y = dest_y;
 	 #endif
  }

}
