/*
 * Projet Système de particules
 *
 * parseur.c
 *
 *  Fonctions pour parser le fichier de description du système de particules
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include "geometrie.h"
#include "list.h"
#include "particule.h"
#include "ancre.h"
#include "systeme.h"
#include "parseur.h"

#define NB_CHAR_LG 255 /* Nombre de caractères par ligne */
#define EPS 1e-8 /* Masse minimale d'une particule */

/** Procédure qui déroule un fichier jusqu'a une ligne sans commentaire
 * Arguments:
 * 	file: fichier à lire
 * 	ligne: variable ou l'ont stocke la ligne
 */
void parse_commentaire(FILE* file, char* ligne){

	while((fgets(ligne, NB_CHAR_LG, file) != NULL)&&(ligne[0]=='#'));

}

/** Procédure lisant la configuration d'un particule avec ajout au système
 * Arguments:
 * 	systeme: Le systeme auquel ajouter la particule
 * 	file: le fichier de configuration
 * 	ligne: la ligne à lire
 * 	id: identifiant de la particule
 */
void lire_particule(Systeme* systeme, FILE* file, char* ligne, int id){
	double x;
	double y;
	double m;
	
	int retour; /* Contient le nombre d'élément lus */

	/* On saute les commentaires */	
	parse_commentaire(file,ligne);
	
	/* Lecture d'une particule, au format
	 * 	x y masse
	 */
	retour = sscanf(ligne,"%lf %lf %lf", &x, &y, &m);
	if(retour != 3) {
	/* Un problème dans le format de la particule */
		fprintf(stderr, "Erreur: Particule %d au format incorrect ! \n",id);
	} else if(m < EPS){
		fprintf(stderr, "Erreur: Particule %d de masse nulle !\n",id);
	} else {
		printf("Particule n° %d de position (%g,%g), de masse %g\n",id,x,y,m);
		
		/* Ajout de la particule au système */
		ajouter_particule(systeme,x,y,m,id);
	}
}

/** Procédure lisant la configuration d'une ancre avec ajout au système
 * Arguments:
 * 	systeme: Le systeme auquel ajouter la particule
 * 	file: le fichier de configuration
 * 	ligne: la ligne à lire
 */
void lire_ancre(Systeme* systeme, FILE* file, char* ligne){
	double x;
	double y;
	int id;

	int retour; /* Contiendra le nombre de données lues */
	
	/* On saute les commentaires */
	parse_commentaire(file,ligne);
	
	/* Lecture d'une ancre au format
	 * 	x y n
	 */	
	retour = sscanf(ligne,"%lf %lf %d", &x, &y, &id);
	
	if(retour != 3) {
		fprintf(stderr, "Erreur dans le format de l'ancre %d ! \n",id);
	} else {
		printf("Ancre de position (%g,%g), associée à la particule %d\n",x,y,id);
	
		/* Ajout de l'ancre au système */
		ajouter_ancre(systeme,x,y,id);
	}
}

/** Procédure lisant la configuration d'une tige avec ajout au système
 * Arguments:
 * 	systeme: Le systeme auquel ajouter la particule
 * 	file: le fichier de configuration
 * 	ligne: la ligne à lire
 */
void lire_tige(Systeme* systeme, FILE* file, char* ligne){
	int id1;
	int id2;
	double l;
	
	int retour; /* Contiendra le nombre de données lues */
	
	/* On saute les commentaires */
	parse_commentaire(file,ligne);
	
	/* Lecture d'une tige au format
	 * 	longueur n1 n2
	 */	
	retour = sscanf(ligne,"%lf %d %d", &l,&id1,&id2);
	if(retour != 3) {
		fprintf(stderr, "Erreur: Tige au format incorrect ! \n");
	} else {
		printf("Tige de longueur %g, associée aux particules %d - %d\n",l,id1,id2);
		
		/* Ajout de la tige au systeme */
		ajouter_tige(systeme,l,id1,id2);
	}
}

/** Procédure lisant la configuration d'une chaine avec ajout au système
 * Arguments:
 * 	systeme: Le systeme auquel ajouter la particule
 * 	file: le fichier de configuration
 * 	ligne: la ligne à lire
 */
void lire_chaine(Systeme* systeme, FILE* file, char* ligne){
	int id1;
	int id2;
	double l;
	
	int retour; /* Variable contenant le nombre de données lues */
		
	/* On saute les commentaires */
	parse_commentaire(file,ligne);
	
	/* Lecture d'une chaine au format
	 * 	longueur n1 n2
	 */	
	retour = sscanf(ligne,"%lf %d %d", &l,&id1,&id2);
	if(retour != 3){
		fprintf(stderr, "Erreur: Chaine de format incorrect !\n");
	} else {
		printf("Chaine de longueur %g, associée aux particules %d - %d\n",l,id1,id2);
		
		/* Ajout de la chaine au systeme */
		ajouter_chaine(systeme,l,id1,id2);
	}
}

/** Procédure lisant la configuration d'un espace avec ajout au système
 * Arguments:
 * 	systeme: Le systeme auquel ajouter l'espace
 * 	file: le fichier de configuration
 * 	ligne: la ligne à lire
 */
void lire_espace(Systeme* systeme, FILE* file, char* ligne){
	int id1;
	int id2;
	double l;
	
	int retour; /* Variable contenant le nombre de données lues */
		
	/* On saute les commenatires */
	parse_commentaire(file,ligne);
	
	/* Lecture d'un espace au format
	 * 	longueur n1 n2
	 */	
	retour = sscanf(ligne,"%lf %d %d", &l,&id1,&id2);
	if(retour != 3){
		fprintf(stderr, "Erreur: Espace de format incorrect !\n");
	} else {
		printf("Espace de longueur %g, associé aux particules %d - %d\n",l,id1,id2);
		
		/* Ajout de l'espace au systeme */
		ajouter_espace(systeme,l,id1,id2);
	}
}

/** Procédure lisant la configuration d'un ressort avec ajout au système
 * Arguments:
 * 	systeme: Le systeme auquel ajouter l'espace
 * 	file: le fichier de configuration
 * 	ligne: la ligne à lire
 */
void lire_ressort(Systeme* systeme, FILE* file, char* ligne){
	int id1;
	int id2;
	double l;
	double k;
	
	int retour; /* Variable contenant le nombre de données lues */
		
	/* On saute les commenatires */
	parse_commentaire(file,ligne);
	
	/* Lecture d'un espace au format
	 * 	longueur coefficient n1 n2
	 */	
	retour = sscanf(ligne,"%lf %lf %d %d", &l,&k,&id1,&id2);
	if(retour != 4){
		fprintf(stderr, "Erreur: Ressort de format incorrect !\n");
	} else {
		printf("Ressort de longueur %g, de coefficient %g, associé aux particules %d - %d\n",l,k,id1,id2);
		
		/* Ajout de l'espace au systeme */
		ajouter_ressort(systeme,l,k,id1,id2);
	}
}
/** Fonction de création du système à partir d'un fichier, passé en paramètre 
 * Argument:
 * 	fichier: Chemin du fichier passé en paramètre.
 * Retourne: Pointeur sur un objet systeme
 */
Systeme* charger(char* fichier) {
	/* Variable stockant la ligne lue */
	char ligne[NB_CHAR_LG];
	
	/* Le système créé */
	Systeme* sys;

	/* Le FILE servant à la lecture du fichier */
	FILE* file;

	/* Les variables servant à parser le fichier */
	double g1,g2,k;
	int nb;
	int i;

	int retour; /* Variable contenant le nombre de données lues */

	/* Ouverture du fichier */
	printf("Ouverture du fichier... %s\n", fichier);
	file = fopen(fichier,"r");

	/* On teste si l'ouverture a réussie */
	if(file != NULL){
		/* Si l'ouverture a réussie */
		puts("Fichier ouvert correctement");
		
		/* On ignore les commentaires */
		parse_commentaire(file, ligne);

		/* Lecture du vecteur gravité */
		retour = sscanf(ligne, "%lf %lf",&g1,&g2);
		if(retour != 2) {
			fprintf(stderr,"Erreur dans la définition de la gravité !\n On prendra les valeurs \"usuelles\"\n");
			g1 = 0.0;
			g2 = 9.8;
		} else {
			printf("Gravité : (%g, %g)\n",g1,g2);
		}

		parse_commentaire(file,ligne);

		/* Lecture du coef de viscosité */
		retour = sscanf(ligne,"%lf",&k);
		if(retour != 1) {
			fprintf(stderr,"Erreur dans le coefficient de friction !\n On prendra une valeur par défaut\"\n");
			k=0.05;
		} else {
			printf("Coef de viscosite: %g\n",k);
		}
		
		/* Création du système */
		puts("Création du système...");
		sys = creer_systeme(g1,g2,k);
		if(sys == NULL){
			fprintf(stderr, "Erreur d'allocation mémoire !\n");
			return NULL;
		}

		parse_commentaire(file,ligne);
		
		/* Lecture du nombre de particules */
		retour = sscanf(ligne,"%d",&nb);
		if(retour != 1){
			fprintf(stderr, "Erreur dans le format du fichier !\n");
			return NULL;
		}
		printf("%d particules\n",nb);

		/* On lit les particules une à une*/
		for(i=0;i<nb;i++){
			lire_particule(sys,file,ligne,i);
		}
		
		parse_commentaire(file,ligne);
		
		/* Lecture du nombre d'ancres */
		retour = sscanf(ligne,"%d",&nb);
		if(retour != 1){
			fprintf(stderr, "Erreur dans le format du fichier\n");
			return NULL;
		}
		printf("%d ancres\n",nb);

		/* On lit les ancres une à une */
		for(i=0;i<nb;i++){
			lire_ancre(sys,file,ligne);
		}
		
		parse_commentaire(file,ligne);
		
		/* Lecture du nombre de tiges */
		retour = sscanf(ligne,"%d",&nb);
		if(retour != 1){
			fprintf(stderr, "Erreur dans le format du fichier !\n");
			return NULL;
		}
		printf("%d tiges\n",nb);

		/* On lit les tiges une à une */
		for(i=0;i<nb;i++){
			lire_tige(sys,file,ligne);
		}

		parse_commentaire(file,ligne);
		
		/* Lecture du nombre de chaine */
		retour = sscanf(ligne,"%d",&nb);
		if(retour != 1){
			fprintf(stderr, "Erreur dans le format du fichier !\n");
			return NULL;
		}
		printf("%d chaines\n",nb);

		/* On lit les chaines une à une */
		for(i=0;i<nb;i++){
			lire_chaine(sys,file,ligne);
		}

		parse_commentaire(file,ligne);
		
		/* Lecture du nombre d'espaces */
		retour = sscanf(ligne,"%d",&nb);
		if(retour != 1){
			fprintf(stderr, "Erreur dans le format du fichier !\n");
			return NULL;
		}
		printf("%d espaces\n",nb);

		/* On lit les espaces un à un */
		for(i=0;i<nb;i++){
			lire_espace(sys,file,ligne);
		}
		
		parse_commentaire(file,ligne);
		
		/* Lecture du nombre de ressorts */
		retour = sscanf(ligne,"%d",&nb);
		if(retour != 1){
			fprintf(stderr, "Erreur dans le format du fichier !\n");
			return NULL;
		}
		printf("%d ressorts\n",nb);

		/* On lit les ressorts un à un */
		for(i=0;i<nb;i++){
			lire_ressort(sys,file,ligne);
		}
		/* À finir : Lecture des contraintes supplémentaires */

		/* Fermeture du fichier */
		fclose(file);

	}else{
		/* En cas d'erreur */
		fprintf(stderr, "Erreur d'Entrée/Sortie à l'ouverture du fichier !\n");
		return NULL;
	}
	
	return sys;
}
