/*
 * Projet Système de particules
 *
 * particule.c
 *
 *  Fonctions et types particule.
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "list.h"
#include "geometrie.h"
#include "particule.h"

/* Précision de la recherche d'une particule par rapport à sa position */
#define DELTA 4

/* Coefficient utilisé lors du calcul de la vitesse */
#define MULT 50

/** Fonction de création d'une particule
 * Arguments:
 * 	x Abscisse de la particule
 * 	y Ordonnée de la particule
 * 	m Masse de la particule
 * Retourne: Un pointeur sur une particule
 */
Particule* creer_particule(double x, double y, double m, double id) {
	/* Allocation de la mémoire */
	Particule* particule = (Particule*)malloc(sizeof(Particule));

	/* Affectation des valeurs */
	particule -> position = creer_point(x,y);
	
	/* La vitesse est donc nulle au premier calcul */
	particule -> position_p = creer_point(x,y);

	/* Aucune force n'est appliquée sur la particule */
	particule -> forces = nil();

	particule -> masse = m;

	particule -> id = id;

	return particule;
}

/** Fonction d'égalité de particules au sens de l'identifiant
 * Teste si deux particules ont même identifiant
 * Arguments:
 * 	p1: Première particule
 * 	p2: Deuxième particule
 */
int egalite_particule_id(Particule *p1, Particule* p2){
	return p1 -> id == p2 -> id;
}

/** Fonction d'égalité de particules au sens des coordonnées
 * Teste si deux particules ont même coordonnées, à +- DELTA
 * Arguments:
 * 	p1: Première particule
 * 	p2: Deuxieme particule
 */
int egalite_particule_pos(Particule* p1, Particule* p2) {
	return (abs(p1 -> position -> x - p2 -> position -> x) <= DELTA) && 
		(abs(p1 -> position -> y - p2 -> position -> y) <= DELTA);
}

/** Fonction de calcul de la vitesse de la particule
 * Arguments:
 * 	particule: Particule dont on souhaite calculer la vitesse
 * 	dt: intervalle de temps
 */
Vecteur* calculer_vitesse (Particule* particule, double dt){
	
	/* On utilise la formule DV = DP/DT */
	double norme_x = particule -> position -> x - particule -> position_p -> x;
	double norme_y = particule -> position -> y - particule -> position_p -> y;

	
	/* Remarque: On utilise MULT car sinon la vitesse tend vers l'infini */
	norme_x /= dt*MULT;
	norme_y /= dt*MULT;
	
	return creer_vecteur(norme_x, norme_y);
}

/** Fonction calculant l'accélération
 * Argument:
 * 	particule: Particule dont on souhaite sommer les forces
 * 	dt: Intervalle de temps entre deux calculs
 * 	gx: Norme X de la force de gravité
 * 	gy: Norme Y de la force de gravité
 * 	k: Coefficient de friction
 * Retourne: Le vecteur accélération (somme des forces / masse)
 */
Vecteur* calculer_acceleration (Particule* particule, double dt, double gx, double gy, double k) {
	
	double nx = 0; /* Composante en X de la somme des forces */
	double ny = 0; /* Composante en Y de la somme des forces */

	Vecteur* vitesse = calculer_vitesse(particule,dt);

	int ret; /* Valeur de retour des fonctions du module liste */

	/* T_LIST qui contiendra la liste des forces */
	T_LIST forces = particule -> forces;
	
	/* Vecteur contenant la tête de la liste des forces */
	Vecteur* vect;

	/* On calcule la somme des forces dues aux ressorts */
	while(!is_empty(forces)){
		/* On récupère la tete de la liste */
		vect = (Vecteur*)head(forces,&ret);

		/* Ajout des valeurs des composantes */
		nx += vect -> norme_x;
		ny += vect -> norme_y;

		/* On passe à la suite de la liste */
		forces = tail(forces, &ret);
	}

	/* Calcul de la force de friction */	
	nx += -k*norme(vitesse)*(vitesse->norme_x);
	ny += -k*norme(vitesse)*(vitesse->norme_y);

	/* On divise par la masse pour obtenir une accélération */
	nx /= particule->masse;
	ny /= particule->masse;
	
	/* On rajoute la gravité */
	nx += gx;
	ny += gy;

	return creer_vecteur(nx,ny);
}


/** Procédure intégrant l'équation du mouvement par l'équation de Verlet
 * Arguments:
 * 	particule: Particule dont on souhaite intégrer le mouvement
 * 	dt: Intervalle de temps
 * 	g: Vecteur gravité gravité
 * 	k: Coefficient de friction
 */
void integrer(Particule* particule, double dt, Vecteur* g, double k){

	double gx = g -> norme_x;
	double gy = g -> norme_y;

	/* On calcule la somme des forces appliquées à cette particule */
	Vecteur* a = calculer_acceleration(particule,dt,gx,gy,k);

	/* On récupère la position courante */
	Point* tmp = particule -> position;

	/* Équation de Verlet */
	double n_x = 2*(tmp -> x) - particule -> position_p -> x + dt*dt*(a -> norme_x);
	double n_y = 2*(tmp -> y) - particule -> position_p -> y + dt*dt*(a -> norme_y);

	/* On crée le point associé à la nouvelle position */
	particule -> position = creer_point(n_x,n_y);

	/* On libère la mémoire de la position précédente */
	free(particule -> position_p);

	/* On met à jour la position précédente */
	particule -> position_p = tmp;

	/* On détruit le vecteur accélération*/
	free(a); a = NULL;

}

/** Procédure ré-initialisant les forces appliquées à une particule
 * Argument: particule: La particule que l'on traite
*/
void raz_forces(Particule* part) {
	/* On applique l'itérateur pour détruire les vecteurs forces stockés
	 * dans la liste des forces
	 */
	do_list(detruire_vecteur_it,part -> forces);

	/* Puis, on réinitialise la liste */
	part -> forces = nil();
}

/** Procédure détruisant la particule
 * Argument: particule: Particule à détruire
 */
void detruire_particule(Particule** particule) {
	/* On détruit chacun de ses champs */
	free((*particule) -> position); (*particule) -> position = NULL;
	free((*particule) -> position_p); (*particule) -> position_p = NULL;
	(*particule) = NULL;
}

/** Procédure utilisée pour convertir l'appel à detruire_particule dans
 * l'itérateur
 * Argument: part: La particule à détruire
 */
void detruire_part_it(Particule* part) {
	detruire_particule(&part);
}
