/*
 * Projet Système de particules
 *
 * particule.h
 *
 *  Fonctions et types particule.
 *
 *
 */

#ifndef _H_PARTICULE
#define _H_PARTICULE

/* Définition du type particule */
typedef struct {
	int id; /* Numéro de création de la particule */
	Point* position; /* La position de la particule en T */ 
	Point* position_p; /* La position de la particule en T-1 */
	double masse; /* Masse de la particule */
	T_LIST forces; /* Les forces appliquées sur la particule */
	} Particule;

/** Fonction de création d'une particule
 * Arguments:
 * 	x Abscisse de la particule
 * 	y Ordonnée de la particule
 * 	m Masse de la particule
 * Retourne: Un pointeur sur une particule
 */
Particule* creer_particule(double x, double y, double m, double id);

/** Fonction d'égalité de particules au sens de l'identifiant
 * Teste si deux particules ont même identifiant
 * Arguments:
 * 	p1: Première particule
 * 	p2: Deuxième particule
 */
int egalite_particule_id(Particule *p1, Particule* p2);

/** Fonction d'égalité de particules au sens des coordonnées
 * Teste si deux particules ont même coordonnées, à +- delta
 * Arguments:
 * 	p1: Première particule
 * 	p2: Deuxieme particule
 */
int egalite_particule_pos(Particule* p1, Particule* p2);

/** Fonction de calcul de la vitesse de la particule
 * Arguments:
 * 	particule: Particule dont on souhaite calculer la vitesse
 * 	dt: intervalle de temps
 * Retourne: Le vecteur vitesse
 */
Vecteur* calculer_vitesse (Particule* particule, double dt);

/** Fonction calculant l'accelération
 * Argument:
 * 	particule: Particule dont on souhaite sommer les forces
 * 	dt: Intervalle de temps entre deux calculs
 * 	gx: Norme X de la force de gravité
 * 	gy: Norme Y de la force de gravité
 * 	k: Coefficient de friction
 * Retourne: Le vecteur accélération (somme des forces / masse)
 */
Vecteur* calculer_acceleration (Particule* particule, double dt, double gx, double gy, double k);

/** Procédure intégrant l'équation du mouvement par l'équation de Verlet
 * Arguments:
 * 	particule: Particule dont on souhaite intégrer le mouvement
 * 	dt: Intervalle de temps
 * 	g: Vecteur gravité
 * 	k: Coefficient de friction
 */
void integrer(Particule* particule, double dt, Vecteur* g, double k);

/** Procédure ré-initialisant les forces appliquées à une particule
 * Argument: particule: La particule que l'on traite
*/
void raz_forces(Particule* part);

/** Procédure détruisant la particule
 * Argument: particule: Particule à détruire
 */
void detruire_particule(Particule** particule);

/** Procédure utilisée pour convertir l'appel à detruire_particule dans
 * l'itérateur
 * Argument: part: La particule à détruire
 */
void detruire_part_it(Particule* part);

#endif
