/*
 * Projet Système de particules
 *
 * ressort.c
 *
 *  Fonctions et type ressort.
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "geometrie.h"
#include "list.h"
#include "particule.h"
#include "ressort.h"

/** Coefficient multiplicateur utilisé lors du calcul de la force exercée par
 * le ressort sur les particules
 */
#define MULT 1

/** Fonction créant un ressort de longueur l, de coefficient k
 *et d'extrémité p1 et p2
 * Arguments:
 * 	p1: Première extrémité
 * 	p2: Seconde extrémité
 * 	l: Longueur de la tige
 * 	k: Coefficient de raideur du ressort
 * Retourne: Un pointeur sur le ressort créé
 */
Ressort* creer_ressort(Particule* p1, Particule* p2, double l, double k) {
	/* Allocation de mémoire */
	Ressort* ressort = (Ressort*)malloc(sizeof(Ressort));

	/* Affectation des champs */
	ressort -> e1 = p1;
	ressort -> e2 = p2;
	ressort -> longueur = l;
	ressort -> coefficient = k;

	return ressort;
}

/** Procédure de calcul ds forces d'appliquant sur chacune des extrémités du
 * ressort
 */
void calculer_force_ressort(Ressort* ressort) {
	/* Principe: On va calculer les forces appliquées sur chacune des
	 * extrémités du ressort, et les rajouter dans la liste de forces de
	 * chacune des particules
	 */

	/* Pour alleger les notations */
	Particule* p1 = ressort -> e1;
	Particule* p2 = ressort -> e2;

	/* Calcul de la distance entre les deux points */
	double dist = distance(p1 -> position, p2 -> position);
	
	/* Coefficient multiplicateur utilisée dans la norme des forces */
	double coef = 
		(dist - ressort -> longueur)*(ressort -> coefficient)/(2*dist*MULT);

	/* Composante en X */
	double c_x = coef*(p2 -> position -> x - p1 -> position -> x);
	/* Composante en Y */
	double c_y = coef*(p2 -> position -> y - p1 -> position -> y);
	
	/* Calcul de f12 */
	Vecteur* f12 = creer_vecteur(c_x,c_y);
	/* Calcul de f21 */
	Vecteur* f21 = creer_vecteur(-c_x,-c_y);

	/* Ajout des forces aux particules */
	cons(f12, &(p1 -> forces));
	cons(f21, &(p2 -> forces));

}

/** Procédure détruisant le ressort
 * Argument: ressort: Ressort à détruire
 */
void detruire_ressort(Ressort** ressort) {
	free(*ressort); (*ressort)=NULL;
}

/** Procédure utilisée pour convertir l'appel à detruire_ressort dans
 * l'itérateur
 * Argument: ressort: Le ressort à détruire
 */
void detruire_ressort_it(Ressort* ressort) {
	detruire_ressort(&ressort);
}
