/*
 * Projet Système de particules
 *
 * systeme.c
 *
 *  Fonctions et types système de particules.
 *
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "list.h"
#include "geometrie.h"
#include "particule.h"
#include "ancre.h"
#include "tige.h"
#include "chaine.h"
#include "espace.h"
#include "ressort.h"
#include "systeme.h"

/* Le nombre d'itérations pour résoudre les contraintes */
#define NB_ITER 100

/* L'intervalle entre chaque appel à la fonction "verlet" */
#define DT 0.5 

/** Fonction qui créant un systeme vide
 * Arguments:
 * 	gx: Norme en X de la gravité
 * 	gy: Norme en Y de la gravité
 * 	k: Coefficient de viscosité
 * Retourne: Pointeur sur un systeme
 */
Systeme* creer_systeme(double gx, double gy, double k) {
	/* On alloue de la mémoire pour le système... */
	Systeme* sys = (Systeme*)malloc(sizeof(Systeme));

	/* Affectation des paramètres du système */
	sys -> coef = k;
	sys -> gravite = creer_vecteur(gx,gy);

	/* Aucune particule n'est déplacée */
	sys -> deplace = NULL;

	/* Les listes sont vides */
	sys -> particules = nil();
	sys -> tiges = nil();
	sys -> ancres = nil();
	sys -> chaines = nil();
	sys -> espaces = nil();
	sys -> ressorts = nil();
	sys -> destructible = 0;

	return sys;
}


/** Procédure créant et ajoutant une particule à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	x: Abscisse de la particule
 * 	y: Ordonnée de la particule
 * 	m: masse de la particule
 * 	id: identifiant de la particule
 */
void ajouter_particule(Systeme* systeme, double x, double y, double m, int id) {
	/* Création de la particule */
	Particule *part = creer_particule(x,y,m,id);

	/* Ajout de la particule à la liste */
	cons(part,&(systeme -> particules));
}

/** Procédure créant et ajoutant une ancre à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	x: Abscisse de l'ancre
 * 	y: Ordonnée
 * 	id: Identifiant de la particule associée
 */
void ajouter_ancre(Systeme* systeme, double x, double y, int id){
	/* Création du particule temporaire servant à la recherche */
	Particule* tmp = creer_particule(0,0,0,id);

	/* On récupère la particule d'identifiant id */
	Particule *part = member(tmp,systeme -> particules, egalite_particule_id);

	/* On vérifie que la particule existe */
	if(part == NULL) {
		fprintf(stderr, "Pas de particule n°%d\n",id);
	} else {
		/* Création de l'ancre */
		Ancre* ancre = creer_ancre(part,x,y);

		/* Ajout de l'ancre au système */
		cons(ancre,&(systeme->ancres));
	}

		
	/* Suppression de la particule */
	detruire_particule(&tmp);
}

/** Procédure créant et ajoutant une tige à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur de la tige
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_tige(Systeme* systeme, double l, int id1, int id2){
	/* Création des particules "motif" */
	Particule* tmp1 = creer_particule(0,0,0,id1);
	Particule* tmp2 = creer_particule(0,0,0,id2);

	/* Recherche des particules par identifiant */
	Particule* part1 = member(tmp1,systeme -> particules,egalite_particule_id);
	Particule* part2 = member(tmp2,systeme -> particules,egalite_particule_id);
	
	/* On vérifie que les particules existent */
	if((part1 == NULL) || (part2 == NULL)){
		fprintf(stderr,"L'une des particules n°%d ou %d n'existe pas\n",
			id1,id2);
	} else {
		/* Création de la tige */
		Tige* tige = creer_tige(part1, part2, l);
		
		/* Ajout de la tige au système */
		cons(tige,&(systeme->tiges));
	}
	/* Suppression des particules temporaires */
	detruire_particule(&tmp1);
	detruire_particule(&tmp2);
}


/** Procédure créant et ajoutant une chaine à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur de la chaine
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_chaine(Systeme* systeme, double l, int id1, int id2){
	/* Création des particules "motif" */
	Particule* tmp1 = creer_particule(0,0,0,id1);
	Particule* tmp2 = creer_particule(0,0,0,id2);

	/* Recherche des particules par identifiant */
	Particule* part1 = member(tmp1,systeme -> particules,egalite_particule_id);
	Particule* part2 = member(tmp2,systeme -> particules,egalite_particule_id);

	/* Test du retour de la fonction member */
	if((part1 == NULL) || (part2 == NULL)){
		fprintf(stderr,"L'une des deux particules n°%d ou %d n'existe pas\n",
			id1, id2);
	} else {
		/* Création de la chaine */
		Chaine* chaine = creer_chaine(part1, part2, l);
		
		/* Ajout de la chaine au système */
		cons(chaine,&(systeme->chaines));
	}

	/* Suppression des particules temporaires */
	detruire_particule(&tmp1);
	detruire_particule(&tmp2);
}


/** Procédure créant et ajoutant un espace à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur de l'espace
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_espace(Systeme* systeme, double l, int id1, int id2){
	/* Création des particules "motif" */
	Particule* tmp1 = creer_particule(0,0,0,id1);
	Particule* tmp2 = creer_particule(0,0,0,id2);

	/* Recherche des particules par identifiant */
	Particule* part1 = member(tmp1,systeme -> particules,egalite_particule_id);
	Particule* part2 = member(tmp2,systeme -> particules,egalite_particule_id);

	/* Test du retour de la fonction member */
	if((part1 == NULL) || (part2 == NULL)){
		fprintf(stderr,"L'une des deux particules n°%d ou %d n'existe pas\n",
			id1, id2);
	} else {
		/* Création de l'espace */
		Espace* Espace = creer_espace(part1, part2, l);
		
		/* Ajout de l'espace au système */
		cons(Espace,&(systeme->espaces));
	}

	/* Suppression des particules temporaires */
	detruire_particule(&tmp1);
	detruire_particule(&tmp2);
}



/** Procédure créant et ajoutant un ressort à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur du ressort
 * 	k: coefficient du ressort
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_ressort(Systeme* systeme, double l, double k, int id1, int id2){
	/* Création des particules "motif" */
	Particule* tmp1 = creer_particule(0,0,0,id1);
	Particule* tmp2 = creer_particule(0,0,0,id2);

	/* Recherche des particules par identifiant */
	Particule* part1 = member(tmp1,systeme -> particules,egalite_particule_id);
	Particule* part2 = member(tmp2,systeme -> particules,egalite_particule_id);
	
	/* On teste si les deux particules existent */
	if((part1 == NULL) || (part2 == NULL)){
		fprintf(stderr,"L'une des particules n°%d ou %d n'existe pas\n",
			id1,id2);
	} else {
		/* Création du ressort */
		Ressort* ressort = creer_ressort(part1, part2, l, k);
		
		/* Ajout du ressort au système */
		cons(ressort,&(systeme->ressorts));
	}

	/* Suppression des particules temporaires */
	detruire_particule(&tmp1);
	detruire_particule(&tmp2);
}


/** Procédure intégrant le systeme
 * Argument: Le système à intégrer
 */
void verlet(Systeme* sys){

	int ret; /* Variable utilisée pour la fonction head */
	
	Particule* part;
	
	/* La liste des particules du systeme */
	T_LIST l_part = sys -> particules;
	
	/* RAZ des forces sur les particules */
	do_list(raz_forces, sys -> particules); 

	/* Calcul des forces exercées par les ressorts */
	do_list(calculer_force_ressort, sys -> ressorts);

	/* On parcourt la liste des particules */
	while(!is_empty(l_part)){
		/* Récupération de la tête de la liste */
		part = (Particule*) head(l_part,&ret);
		/* On calcule la nouvelle position de la particule */
		integrer(part, DT, sys -> gravite, sys -> coef);
		/* On récupère le reste de la liste */
		l_part = tail(l_part,&ret);
	}

}

/** Procédure résolvant les contraintes
 * Argument: Le système dont on souhaite satisfaire les contraintes
 */
void satisfaire_contraintes(Systeme* sys) {
	
	int i;


	/* Résolution des contraintes */
	for(i=0;i<NB_ITER;i++){
		
		/* On résout d'abord les contraintes sur les tiges */
		do_list(resoudre_contrainte_tige, sys -> tiges);
		/* Puis sur les chaines */ 
		do_list(resoudre_contrainte_chaine, sys -> chaines);
		/* Puis les espaces */
		do_list(resoudre_contrainte_espace, sys -> espaces);
		/* Puis sur les ancres */
		do_list(resoudre_contrainte_ancre, sys -> ancres);
		
		/* Si le système contient une ancre en train d'être déplacée, il faut
		 * aussi la traiter dans les contraintes */
		if(sys -> deplace != NULL){
			resoudre_contrainte_ancre(sys -> deplace);
		}
	}
	
}

/** Procédure détruisant le système
 * Argument: Système à détruire
 */
void detruire_systeme(Systeme* sys) {

	/* On commence par détruire chacun des objets du systeme */
	do_list(detruire_ancre_it,sys -> ancres);
	do_list(detruire_tige_it,sys -> tiges);
	do_list(detruire_chaine_it, sys -> chaines);
	do_list(detruire_espace_it, sys -> espaces);
	do_list(detruire_part_it, sys -> particules);
	do_list(detruire_ressort_it, sys -> ressorts);

	/* On libère la mémoire du système */
	free(sys);

	printf("Système détruit !\n");
}
