/*
 * Projet Système de particules
 *
 * systeme.h
 *
 *  Fonctions et types système de particules.
 *
 *
 */

#ifndef _H_SYSTEME
#define _H_SYSTEME

/* Définition du type Systeme */
typedef struct {

	/* Les paramètres physiques du systeme */
	Vecteur* gravite; /* Le vecteur gravité */

	double coef; /* Coefficient de viscosité */

	/* La liste des éléments du système */
	T_LIST particules; /* Liste de particules */

	T_LIST tiges; /* Liste de tiges */

	T_LIST ancres; /* Liste d'ancres */

	T_LIST chaines; /* Liste de chaines */

	T_LIST espaces; /* Liste d'espaces */

	T_LIST ressorts; /* Liste de ressorts */
	
	/* Les données nécessaires pour l'interaction avec la souris */
	Ancre* deplace; /* Ancre qui sera déplacée par la souris */

	int destructible; /* Indique si l'ancre précédente peut être détruite */

	} Systeme;


/** Fonction qui créant un systeme vide
 * Arguments:
 * 	gx: Norme en X de la gravité
 * 	gy: Norme en Y de la gravité
 * 	k: Coefficient de viscosité
 * Retourne: Pointeur sur un systeme
 */
Systeme* creer_systeme(double gx, double gy, double k);

/** Procédure créant et ajoutant une particule à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	x: Abscisse de la particule
 * 	y: Ordonnée de la particule
 * 	m: masse de la particule
 * 	id: identifiant de la particule
 */
void ajouter_particule(Systeme* systeme, double x, double y, double m, int id);

/** Procédure créant et ajoutant une ancre à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	x: Abscisse de l'ancre
 * 	y: Ordonnée
 * 	id: Identifiant de la particule associée
 */
void ajouter_ancre(Systeme* systeme, double x, double y, int id);

/** Procédure créant et ajoutant une tige à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur de la tige
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_tige(Systeme* systeme, double l, int id1, int id2);

/** Procédure créant et ajoutant une chaine à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur de la chaine
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_chaine(Systeme* systeme, double l, int id1, int id2);

/** Procédure créant et ajoutant un espace à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur de l'espace
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_espace(Systeme* systeme, double l, int id1, int id2);

/** Procédure créant et ajoutant un ressort à un système
 * Arguments:
 * 	systeme: Le systeme que l'on modifie
 * 	l: longueur du ressort
 * 	k: coefficient du ressort
 * 	id1: Identifiant de la première particule
 * 	id2: Identifiant de la seconde particule
 */
void ajouter_ressort(Systeme* systeme, double l, double k, int id1, int id2);

/** Procédure intégrant le systeme
 * Argument: Le système à intégrer
 */
void verlet(Systeme* sys);

/** Procédure résolvant les contraintes
 * Argument: Le système dont on souhaite satisfaire les contraintes
 */
void satisfaire_contraintes(Systeme* sys);

/** Procédure détruisant le système
 * Argument: Système à détruire
 */
void detruire_systeme(Systeme* sys);

#endif
