/*
 * Projet Système de particules
 *
 * tige.c
 *
 *  Fonctions et type tige.
 *
 *
 */
#include <stdlib.h>
#include <math.h>
#include "list.h"
#include "geometrie.h"
#include "particule.h"
#include "tige.h"

/** Précision utilisée lors de la recherche d'une particule par rapport à sa
 * position
*/
#define DELTA 4

/** Fonction créant une tige de longueur l et d'extrémité p1 et p2
 * Arguments:
 * 	p1: Première extrémité de la tige
 * 	p2: Seconde extrémité de la tige
 * 	l: Longueur de la tige
 * Retourne: Un pointeur sur la tige créée
 */
Tige* creer_tige(Particule* p1, Particule* p2, double l) {
	/* Allocation de mémoire */
	Tige* tige = (Tige*)malloc(sizeof(Tige));

	/* Affectation des valeurs */
	tige -> e1 = p1;
	tige -> e2 = p2;
	tige -> longueur = l;

	return tige;
}

/** Procédure de résolution de contrainte de type "tige"
 * Argument: La tige dont on veut résoudre la contrainte
 * Algorithme basé sur celui décruit sur le site suivant : 
 * 	www.teknikus.dk/tj/gdc2001.htm
 */
void resoudre_contrainte_tige(Tige* tige) {
	Point* pt1 = tige -> e1 -> position;
	Point* pt2 = tige -> e2 -> position;

	/* On récupère l'inverse des masses des particules */
	double invm1 = tige -> e1 -> masse;
	double invm2 = tige -> e2 -> masse; 

	/* On calcule la distance entre les deux points */
	double delta_x = pt2 -> x - pt1 -> x;
	double delta_y = pt2 -> y - pt1 -> y;

	/* delta est la norme du vecteur pt2 - pt1 */
	double delta = sqrt(pow(delta_x,2)+pow(delta_y,2));

	/* La correction à appliquer aux points pour donner longueur fixée */
	double diff = (delta - tige -> longueur)/(delta*(invm1 + invm2));

	/* On corrige le premier point */
	pt1 -> x += invm1*delta_x*diff;
	pt1 -> y += invm1*delta_y*diff;

	/* On corrige le second point */
	pt2 -> x -= invm2*delta_x*diff;
	pt2 -> y -= invm2*delta_y*diff;
}

/** Procédure détruisant la tige
 * Argument: tige: Tige à détruire
 */
void detruire_tige(Tige** tige) {
	/* Les particules seront supprimées dans une autre procédure */
	free(*tige); (*tige) = NULL;
}

/** Procédure utilisée pour convertir l'appel à detruire_tigee dans
 * l'itérateur
 * Argument: tige: La tige à détruire
 */
void detruire_tige_it(Tige* tige) {
	detruire_tige(&tige);
}

/** Fonction d'égalité de deux tiges par la position d'une de leur extrémité
 * Arguments:
 * 	t1: Première tige
 * 	t2: Seconde tige
 * Retourne: (extremite 1 de t1 == extremite 1 de t2) || (extremite 2 t1 ==
 * 							extremite 2 de t2)
 */
int egalite_tige_ext(Tige* t1, Tige* t2) {
	return (egalite_particule_pos(t1 -> e1, t2 -> e1) || egalite_particule_pos(t1 -> e2, t2 -> e2));
}
