/*
 * Programme principal.
 * Lance une fenetre de visualisation et d'interaction pour la
 * manipulation du système de particules
 *
 *
 */

#include <gtk/gtk.h>
#include <locale.h>
#include "interaction.h"

#include "list.h"
#include "geometrie.h"
#include "particule.h"
#include "ancre.h"
#include "tige.h"
#include "chaine.h"
#include "espace.h"
#include "ressort.h"
#include "systeme.h"
#include "parseur.h"


/* Taille des divers éléments dessinés */
#define TAILLE_PARTICULE 4
#define TAILLE_ANCRE 6
#define TAILLE_TIGE 3
#define TAILLE_RESSORT 5

#define VIEWPORT_WIDTH   400
#define VIEWPORT_HEIGHT  300

typedef struct {
  GtkWidget *viewport;
  GdkPixmap *image;
  GdkGC *gc;
  GdkColor color;
  gboolean receive_drag;
  gdouble drag_orig_x;
  gdouble drag_orig_y;
} Visualiseur;

static Visualiseur visualiseur;

/* Pointeur global sur le type Systeme */
Systeme* systeme;

/************************************
 *
 * Interaction avec la souris
 *
 ************************************/

void viewport_activate_drag(int activate) {
  visualiseur.receive_drag = activate;
}

static gboolean on_viewport_motion(GtkWidget      *widget,
                                   GdkEventMotion  *event,
                                   Visualiseur     *visualiseur) {
  gdouble x, y;

  x = event->x;
  y = event->y;

  if (visualiseur->receive_drag) {
    on_viewport_mouse_drag (visualiseur->drag_orig_x, visualiseur->drag_orig_y,
        x, y);
  }

  return TRUE;
}

static gboolean on_viewport_button_press(GtkWidget      *widget,
                                         GdkEventButton *event,
                                         Visualiseur    *visualiseur) {
  gdouble x, y;

  x = event->x;
  y = event->y;

  visualiseur->drag_orig_x = x;
  visualiseur->drag_orig_y = y;

  on_viewport_mouse_press (x, y);

  return TRUE;
}

static gboolean on_viewport_button_release(GtkWidget      *widget,
                                           GdkEventButton *event,
                                           gpointer        user_data) {
  gdouble x, y;

  x = event->x;
  y = event->y;

  on_viewport_mouse_release (x, y);

  return TRUE;
}

/********************************************
 *
 * Fonctions de dessin
 *
 *******************************************/

void viewport_draw_point(double x, double y, gint size) {

  GdkDrawable *drawable = visualiseur.image;
  GdkGC *gc = visualiseur.gc;

  gdk_draw_arc (drawable, gc,
      TRUE, (gint)x-size/2, (gint)y-size/2,
      size, size,
      0, 64*360);

  gtk_widget_queue_draw (visualiseur.viewport);
}

void viewport_draw_line(double x1, double y1, double x2, double y2, int width) {

  GdkDrawable *drawable = visualiseur.image;
  GdkGC *gc = visualiseur.gc;

  gdk_gc_set_line_attributes (gc, width, 
      GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);	
  gdk_draw_line (drawable, gc, 
      (gint)x1, (gint)y1, (gint)x2, (gint)y2);

  gtk_widget_queue_draw (visualiseur.viewport);
}

void viewport_draw_polygon(Point *points, int nb_points, int plein, int line_width) {

  GdkDrawable *drawable = visualiseur.image;
  GdkGC *gc = visualiseur.gc;
  GdkPoint *gdk_points;
  gint i;

  gdk_gc_set_line_attributes (gc, line_width, 
      GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);	    

  gdk_points = g_new (GdkPoint, nb_points);
  for (i=0; i<nb_points; i++) {
    gdk_points[i].x = (gint)points[i].x;
    gdk_points[i].y = (gint)points[i].y;
  }

  gdk_draw_polygon (drawable, gc, plein, gdk_points, nb_points);

  g_free (gdk_points);

  gtk_widget_queue_draw (visualiseur.viewport);

}

void viewport_change_color(double red, double green, double blue) {

  GdkGC *gc = visualiseur.gc;

  visualiseur.color.red   = CLAMP (red * 0xFFFF, 0, 0xFFFF);
  visualiseur.color.green = CLAMP (green * 0xFFFF, 0, 0xFFFF);
  visualiseur.color.blue  = CLAMP (blue * 0xFFFF, 0, 0xFFFF);
  gdk_gc_set_rgb_fg_color (gc, &visualiseur.color);
}

void viewport_clear (){
  GdkDrawable *drawable = visualiseur.image;
  GdkGC *gc = visualiseur.gc;
  GdkColor white;
  gint width, height;

  /* change color to white */
  white.red = white.green = white.blue = 0xFFFF;
  gdk_gc_set_rgb_fg_color (gc, &white);

  gdk_drawable_get_size (GDK_DRAWABLE (visualiseur.image),
      &width, &height);
  gdk_draw_rectangle  (drawable, gc,
      TRUE,
      0, 0, width, height);

  /* change back to user color */
  gdk_gc_set_rgb_fg_color (gc, &visualiseur.color);

  gtk_widget_queue_draw (visualiseur.viewport);
}

/** Procédure affichant à l'écran une particule
 * Argument:
 * 	p: La particule à afficher
 */
void dessiner_particule(Particule* p){
	viewport_change_color(0,0,0);
	viewport_draw_point(p->position->x,p->position->y,TAILLE_PARTICULE);
}

/** Procédure affichant à l'écran une ancre
 * Argument:
 * 	a: Ancre à afficher
 */
void dessiner_ancre(Ancre* a){
	Particule* p = a->particule;
	viewport_change_color(1,0.2,0.2);
	viewport_draw_point(p->position->x,p->position->y,TAILLE_ANCRE);
}

/** Procédure affichant à l'écran une tige
 * Argument:
 * 	t: La tige à afficher
 */
void dessiner_tige(Tige* t){
	Particule* p1 = t->e1;
	Particule* p2 = t->e2;
	viewport_change_color(0.4,1,0.4);
	viewport_draw_line(p1->position->x,p1->position->y,p2->position->x,p2->position->y,TAILLE_TIGE);
}

/** Procédure affichant à l'écran une chaine
 * Argument:
 * 	c: La chaine à afficher
 */
void dessiner_chaine(Chaine* c){
	Particule* p1 = c->e1;
	Particule* p2 = c->e2;
	viewport_change_color(1,0.8,0.5);
	viewport_draw_line(p1->position->x,p1->position->y,p2->position->x,p2->position->y,TAILLE_TIGE);
}

/** Procédure affichant à l'écran un espace
 * Argument:
 * 	e: L'espace à afficher
 */
void dessiner_espace(Espace* e){
	Particule* p1 = e->e1;
	Particule* p2 = e->e2;
	viewport_change_color(0.2,0.5,1);
	viewport_draw_line(p1->position->x,p1->position->y,p2->position->x,p2->position->y,TAILLE_TIGE);
}

/** Procédure affichant à l'écran un ressort
 * Argument:
 * 	r: le ressort
 */
void dessiner_ressort(Ressort* r){
	Particule* p1 = r->e1;
	Particule* p2 = r->e2;
	viewport_change_color(0.2,0,0.5);
	viewport_draw_line(p1->position->x,p1->position->y,p2->position->x,p2->position->y,TAILLE_RESSORT);
}

/******************************************************************/

gboolean animation_callback(gpointer unused) {

  /* On efface tout */
  viewport_clear();

  /* Calcul des nouvelles positions */
  verlet(systeme);
  satisfaire_contraintes(systeme);


  /* Affichage du système */
  do_list(dessiner_tige,systeme->tiges);
  do_list(dessiner_chaine,systeme->chaines);
  do_list(dessiner_espace,systeme->espaces);
  do_list(dessiner_ressort,systeme->ressorts);
  do_list(dessiner_particule,systeme->particules);
  do_list(dessiner_ancre,systeme->ancres);

  return TRUE;
}

/******************************************************************/

gint main(gint argc, gchar **argv) {

  GtkWidget *window;
  GtkWidget *viewport;
  GtkWidget *event_box;
  GdkPixmap *image;

  gtk_init (&argc, &argv);

  /* Réglage des locales */
  setlocale(LC_ALL, "C");

  g_print ("GTK version :%d.%d.%d\n",
      GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Système de particules");
  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
  g_signal_connect (G_OBJECT (window), "delete-event",
      G_CALLBACK (gtk_main_quit), NULL);

  image = gdk_pixmap_new (NULL, VIEWPORT_WIDTH, VIEWPORT_HEIGHT, 24);
  viewport = gtk_image_new_from_pixmap (image, NULL);
  gtk_misc_set_alignment (GTK_MISC (viewport), 0.0, 0.0);
  gtk_widget_set_size_request (viewport, VIEWPORT_WIDTH, VIEWPORT_HEIGHT);

  event_box = gtk_event_box_new ();
  g_signal_connect (G_OBJECT (event_box), "motion-notify-event",
      G_CALLBACK (on_viewport_motion), &visualiseur);
  g_signal_connect (G_OBJECT (event_box), "button-press-event",
      G_CALLBACK (on_viewport_button_press), &visualiseur);
  g_signal_connect (G_OBJECT (event_box), "button-release-event",
      G_CALLBACK (on_viewport_button_release), &visualiseur);

  gtk_container_add (GTK_CONTAINER (event_box), viewport);
  gtk_container_add (GTK_CONTAINER (window), event_box);

  gtk_widget_show_all (window);

  viewport_activate_drag (TRUE);
  visualiseur.viewport = viewport;
  visualiseur.image = image;
  visualiseur.gc = gdk_gc_new (image);

  viewport_clear ();

  /* -------------------*/

  /* Création du système */
  if(argc == 2){
	  systeme = charger(argv[1]);
  } else {
	systeme = charger("corde.sp");
  }

  /* On vérifie qu'il n'y a pas eu de problème à la création du système */
  if(systeme == NULL) {
	  return -1;
  }

  /* la fonction animation_callback est appelée toutes les 50ms */
  g_timeout_add (50, (GSourceFunc)animation_callback, NULL);

  /* -------------------*/

  gtk_main ();
  
  /* -------------------*/
  /* Destruction du système */
  detruire_systeme(systeme); 

  return 0;
}

